@@ -1,12 +1,12 @@
 
 import numpy as np
+from pyrr import Matrix44, matrix44
 
 from ..io import read_mesh_file
 from ..io.voxels import read_binvox
+from ..utils import normalize_colors
 from .base import Renderable
 
-from pyrr import Matrix44, matrix44
-
 
 class MeshBase(Renderable):
     """Abstract base class that implements functions commonly used by the
@@ -21,6 +21,27 @@ class MeshBase(Renderable):
         self._vbo = None
         self._vao = None
 
+    def to_points_and_faces(self):
+        N = 0
+        index = {}
+        faces = []
+        for x in self._vertices.reshape(-1, 3):
+            x = tuple(x)
+            if x not in index:
+                index[x] = N
+                N += 1
+            faces.append(index[x])
+        points = [p for p, i in sorted(index.items(), key=lambda x: x[1])]
+
+        return np.array(points), np.array(faces).reshape(-1, 3)
+
+    @property
+    def vertices(self):
+        """Return all the vertices of this mesh. The vertices in groups of 3
+        form triangles and several vertices occur multiple times as they are
+        shared by several triangles."""
+        return np.copy(self._vertices)
+
     @property
     def bbox(self):
         """The axis aligned bounding box of all the vertices as two
@@ -98,6 +119,11 @@ class MeshBase(Renderable):
         self._vertices = self._vertices.dot(R) + t
         self._update_vbo()
 
+    def translate(self, t):
+        """Translate all the vertices with a vector t."""
+        self._vertices += t
+        self._update_vbo()
+
     def to_unit_cube(self):
         """Transform the mesh such that it fits in the 0 centered unit cube."""
         bbox = self.bbox
@@ -110,6 +136,9 @@ class MeshBase(Renderable):
         self._prog.release()
         self._vbo.release()
         self._vao.release()
+        self._prog = None
+        self._vbo = None
+        self._vao = None
 
     def render(self):
         self._vao.render()
@@ -147,8 +176,11 @@ class Mesh(MeshBase):
         offset: A translation vector for all the vertices. It can be changed
                 after construction to animate the object together with the
                 `model_matrix` property.
+        mode: {"shading", "flat", "depth"} A string that defines the rendering
+              mode for this mesh. (default: shading)
     """
-    def __init__(self, vertices, normals, colors, offset=[0, 0, 0.]):
+    def __init__(self, vertices, normals, colors, offset=[0, 0, 0.],
+                 mode="shading"):
         super(Mesh, self).__init__(vertices, normals, offset)
 
         self._colors = np.asarray(colors)
@@ -161,6 +193,8 @@ class Mesh(MeshBase):
         elif self._colors.shape[1] == 3:
             self._colors = np.hstack([self._colors, np.ones((N, 1))])
 
+        self._mode = mode
+
     def init(self, ctx):
         self._prog = ctx.program(
             vertex_shader="""
@@ -199,6 +233,7 @@ class Mesh(MeshBase):
                 #version 330
 
                 uniform vec3 light;
+                uniform int mode;
                 in vec3 v_vert;
                 in vec3 v_norm;
                 in vec4 v_color;
@@ -210,7 +245,21 @@ class Mesh(MeshBase):
                     lum = acos(lum) / 3.14159265;
                     lum = clamp(lum, 0.0, 1.0);
 
-                    f_color = vec4(v_color.xyz * lum, v_color.w);
+                    // shading
+                    if (mode == 0) {
+                        f_color = vec4(v_color.xyz * lum, v_color.w);
+                    }
+
+                    // flat color
+                    else if (mode == 1) {
+                        f_color = v_color;
+                    }
+
+                    // depth with orthographic projection
+                    else {
+                        float depth = 1-gl_FragCoord.z;
+                        f_color = vec4(depth, depth, depth, 1);
+                    }
                 }
             """
         )
@@ -226,6 +275,18 @@ class Mesh(MeshBase):
         )
         self.model_matrix = self._model_matrix
         self.offset = self._offset
+        self.mode = self._mode
+
+    @property
+    def colors(self):
+        """Return the color per vertex."""
+        return self._colors.clone()
+
+    @colors.setter
+    def colors(self, c):
+        c = normalize_colors(c, len(self._vertices))
+        self._colors = c
+        self._update_vbo()
 
     def _get_uniforms_list(self):
         """Return the used uniforms to fetch from the scene."""
@@ -239,6 +300,21 @@ class Mesh(MeshBase):
                 self._vertices, self._normals, self._colors
             ]).astype(np.float32).tobytes())
 
+    @property
+    def mode(self):
+        return self._mode
+
+    @mode.setter
+    def mode(self, new_mode):
+        self._mode = new_mode
+        modes = dict(
+            shading=0,
+            flat=1,
+            depth=2
+        )
+        if self._prog:
+            self._prog["mode"] = modes[self._mode]
+
     def sort_triangles(self, point):
         """Sort the triangles such that the first is furthest from `point` and
         the last is the closest to `point`.
@@ -259,24 +335,6 @@ class Mesh(MeshBase):
         self._colors = colors[idxs].reshape(-1, 4)
         self._update_vbo()
 
-    def scale(self, s):
-        """Multiply all the vertices with a number s."""
-        self._vertices *= s
-        self._update_vbo()
-
-    def translate(self, t):
-        """Translate all the vertices with a vector t."""
-        self._vertices += t
-        self._update_vbo()
-
-    def to_unit_cube(self):
-        """Transform the mesh such that it fits in the 0 centered unit cube."""
-        bbox = self.bbox
-        dims = bbox[1] - bbox[0]
-        self._vertices -= dims/2 + bbox[0]
-        self._vertices /= dims.max()
-        self._update_vbo()
-
     @classmethod
     def from_file(cls, filepath, color=(0.3, 0.3, 0.3), ext=None):
         """Read the mesh from a file.
@@ -353,20 +411,20 @@ class Mesh(MeshBase):
         return cls(vertices, normals, colors)
 
     @classmethod
-    def from_faces(cls, vertices, faces, colors):
+    def from_faces(cls, vertices, faces, colors=(0.3,0.3,0.3)):
         vertices, faces, colors = list(map(
             np.asarray,
             [vertices, faces, colors]
         ))
+        colors = normalize_colors(colors, len(vertices))
         vertices = vertices[faces].reshape(-1, 3)
         normals = np.repeat(cls._triangle_normals(vertices), 3, axis=0)
-        if len(colors.shape) != 1:
-            colors = colors[faces].reshape(-1, 3)
+        colors = colors[faces].reshape(-1, 4)
 
         return cls(vertices, normals, colors)
 
     @classmethod
-    def from_boxes(cls, centers, sizes, colors):
+    def from_boxes(cls, centers, sizes, colors=(0.3,0.3,0.3)):
         """Create boxes.
 
         Arguments
@@ -478,7 +536,7 @@ class Mesh(MeshBase):
 
         Arguments
         ---------
-            voxels: Array of 3D values, with truthy values indicating which
+            voxels: Array of 3D values, with true values indicating which
                     voxels to fill
             colors: The colors of the voxels. If colors is a vector then
                     it is the same for all voxels. If it is a 4 dimensional
